import logging
import os

logging.basicConfig(level=logging.INFO)
logger = logging.getLogger(__name__)


def write_file(path: str, content: str, filename: str):
    full_path = os.path.join(path, filename)
    dir_path = os.path.dirname(full_path)
    if not os.path.exists(dir_path):
        os.makedirs(dir_path, exist_ok=True)
        logger.info(f"Created directory: {dir_path}")

    logger.info(f"Writing file to {full_path}")
    with open(full_path, "w", encoding="utf-8") as f:
        f.write(content)


if __name__ == "__main__":
    path = "/home/agga/Documents/odoo-dev/ica_standard_structure/test"
    content = "Hello, this is a test."
    filename = "test.txt"
    write_file(path, content, filename)



from odooss.utils import write_file


def create_ruff(path: str):
    content = """\
[lint]
select = [
    "E",    # pycodestyle
    "F",    # Pyflakes
    "W",    # pycodestyle warnings
    "N",    # pep8-naming (naming conventions)
    "B",    # flake8-bugbear (code smells & unsafe patterns)
    "T201", # print()
    "UP",   # pyupgrade (old syntax like Python 2-style code)
    "PLC",  # pylint-convention
    "PLE",  # pylint-error
    "PLR",  # pylint-refactor
    "PLW",  # pylint-warning
]
fixable = ["ALL"]


ignore = [
    "E501", # line too long
    "E301", # expected 1 blank line, found 0
    "E302", # expected 2 blank lines, found 1
    "PLR0913", #  Too many arguments
]

[lint.per-file-ignores]
"__init__.py" = [
    "F401", # import but unused
]
"__manifest__.py"=[
    "B018",# useless expression.
]
"""
    write_file(path, content, filename="ruff.toml")


if __name__ == "__main__":
    path = "/home/agga/Documents/odoo-dev/ica_standard_structure/test"
    version = 18.0
    python = 3.10
    create_ruff(path)



from .docker import create_docker_compose, create_dockerfile, create_dockerignore
from .gitignore import create_gitignore
from .requirements import create_requirement
from .ruff import create_ruff
from .tools import create_addons, create_readme

__all__ = [
    "create_readme",
    "create_addons",
    "create_requirement",
    "create_ruff",
    "create_gitignore",
    "create_dockerignore",
    "create_dockerfile",
    "create_docker_compose",
]



from odooss.utils import write_file


def create_github_workflow(path:str):
    content="""\
    name: Odoo Modules Update with Docker Compose

on:
  push:
    branches:
      - main

jobs:
  update-modules:
    runs-on: ubuntu-latest

    steps:
      - name: Checkout code
        uses: actions/checkout@v3

      - name: Create odoo_pg_pass secret file
        run: echo "odoo" > odoo_pg_pass

      - name: Create config directory and odoo.conf
        run: |
          mkdir -p config
          cat > config/odoo.conf <<EOF
          [options]
          ; This is the Odoo configuration file
          admin_passwd=master
          db_name=odoo
          addons_path = /mnt/extra-addons
          EOF

      - name: Start Docker Compose
        run: docker compose up -d

      - name: Wait for Odoo to be ready
        run: |
          echo "Waiting for Odoo to start..."
          until docker compose exec web curl -s http://localhost:8069 > /dev/null; do
            echo -n "."
            sleep 5
          done
          echo "Odoo is ready."

      - name: Get latest commit message
        id: get_commit
        run: echo "message=$(git log -1 --pretty=%B)" >> $GITHUB_OUTPUT

      - name: Extract modules from commit message
        id: extract_modules
        run: |
          echo "Commit message: ${{ steps.get_commit.outputs.message }}"
          modules=$(echo "${{ steps.get_commit.outputs.message }}" | grep -oP '(?<=#odoo-module:)[\w,]+')
          echo "modules=$modules" >> $GITHUB_OUTPUT

      - name: Update Odoo modules inside container
        if: steps.extract_modules.outputs.modules != ''
        run: |
          echo "Updating modules: ${{ steps.extract_modules.outputs.modules }}"
          docker compose run web odoo -u ${{ steps.extract_modules.outputs.modules }} -d odoo --stop-after-init --no-http

      - name: Shutdown Docker Compose
        if: always()
        run: docker compose down
    """
    write_file(path, content, filename=".github/ci_cd.yml")

if __name__ == "__main__":
    path = "/home/agga/Documents/odoo-dev/ica_standard_structure/test"
    create_github_workflow(path)



import subprocess
import sys

from odooss.utils import write_file
import logging

logging.basicConfig(level=logging.INFO)
logger = logging.getLogger(__name__)


def create_requirement(path: str):
    content = """\
ruff>=0.12.8
pre-commit>=3.8.0
"""
    file_name = "requirements.txt"
    write_file(path, content, filename=file_name)

    try:
        # Run pip via current Python interpreter
        subprocess.run(
            [sys.executable, "-m", "pip", "install", "-r", file_name], cwd=path, check=True
        )
    except subprocess.CalledProcessError:
        logger.info('you need to install requirements.txt first.')


if __name__ == "__main__":
    path = "/home/agga/Documents/odoo-dev/ica_standard_structure/test"
    create_requirement(path)



import logging

from .folder_structure import (
    create_addons,
    create_docker_compose,
    create_dockerfile,
    create_dockerignore,
    create_gitignore,
    create_readme,
    create_requirement,
    create_ruff,
)
from .precommit import create_precommit_config, install_precommit
from .vcs import create_github_workflow
logging.basicConfig(level=logging.INFO)
logger = logging.getLogger(__name__)


def create_all(path, odoo_version, python,vcs):
    """Create the Odoo standard folder structure at PATH."""
    logger.info("Please make sure your virtual environment is activated.")
    create_dockerfile(path, odoo_version=odoo_version, python=python)
    create_dockerignore(path)
    create_docker_compose(path)

    create_gitignore(path)
    create_requirement(path)
    create_ruff(path)
    create_readme(path)
    create_addons(path)
    create_precommit_config(path)
    install_precommit(path)
    if vcs == 'github':
        create_github_workflow(path)


if __name__ == "__main__":
    path = "/home/agga/Documents/odoo-dev/ica_standard_structure/test"
    version = 18.0
    python = 3.10
    create_all(path, version, python)



from odooss.utils import write_file


def create_dockerfile(path: str, odoo_version: float, python: float):
    content = f"""# Stage 1: Build Dependencies
FROM python:{python}-slim AS builder

WORKDIR /opt/odoo

# Copy and install Python dependencies
COPY ./requirements.txt .
RUN pip3 install --no-cache-dir -r requirements.txt

# Stage 2: Final Image with Odoo
FROM odoo:{odoo_version}

USER root

# Copy only necessary files from the builder stage
COPY --from=builder /opt/odoo /opt/odoo

# Copy custom addons
COPY ./addons /mnt/extra-addons

USER odoo
"""
    write_file(path, content, filename="Dockerfile")


def create_docker_compose(path):
    content = """\
version: '3.1'
services:
  web:
    build:
      context: .
    depends_on:
      - db
    ports:
      - "8069:8069"
    volumes:
      - odoo-web-data:/var/lib/odoo
      - ./config:/etc/odoo
    environment:
      - PASSWORD_FILE=/run/secrets/postgresql_password
    secrets:
      - postgresql_password
  db:
    image: postgres:15
    environment:
      - POSTGRES_DB=postgres
      - POSTGRES_PASSWORD_FILE=/run/secrets/postgresql_password
      - POSTGRES_USER=odoo
      - PGDATA=/var/lib/postgresql/data/pgdata
    volumes:
      - odoo-db-data:/var/lib/postgresql/data/pgdata
    secrets:
      - postgresql_password
volumes:
  odoo-web-data:
  odoo-db-data:

secrets:
  postgresql_password:
    file: odoo_pg_pass
    """
    write_file(path, content, filename="compose.yml")


def create_dockerignore(path: str):
    content = """\
# Include any files or directories that you don't want to be copied to your
# container here (e.g., local build artifacts, temporary files, etc.).
#
# For more help, visit the .dockerignore file reference guide at
# https://docs.docker.com/go/build-context-dockerignore/

**/.DS_Store
**/__pycache__
**/.venv
**/.classpath
**/.dockerignore
**/.env
**/.git
**/.gitignore
**/.project
**/.settings
**/.toolstarget
**/.vs
**/.vscode
**/*.*proj.user
**/*.dbmdl
**/*.jfm
**/bin
**/charts
**/docker-compose*
**/compose.y*ml
**/Dockerfile*
**/node_modules
**/npm-debug.log
**/obj
**/secrets.dev.yaml
**/values.dev.yaml
LICENSE
README.md
"""
    write_file(path, content, filename=".dockerignore")


if __name__ == "__main__":
    path = "/home/agga/Documents/odoo-dev/ica_standard_structure/test"
    version = 18.0
    python = 3.10
    create_dockerfile(path, version, python)
    create_docker_compose(path)
    create_dockerignore(path)



from odooss.utils import write_file


def create_gitignore(path: str):
    content = """\
# Byte-compiled / optimized / DLL files
__pycache__/
*.py[codz]
*$py.class

# C extensions
*.so

# Distribution / packaging
.Python
build/
develop-eggs/
dist/
downloads/
eggs/
.eggs/
lib/
lib64/
parts/
sdist/
var/
wheels/
share/python-wheels/
*.egg-info/
.installed.cfg
*.egg
MANIFEST

# PyInstaller
#  Usually these files are written by a python script from a template
#  before PyInstaller builds the exe, so as to inject date/other infos into it.
*.manifest
*.spec

# Installer logs
pip-log.txt
pip-delete-this-directory.txt

# Unit test / coverage reports
htmlcov/
.tox/
.nox/
.coverage
.coverage.*
.cache
nosetests.xml
coverage.xml
*.cover
*.py.cover
.hypothesis/
.pytest_cache/
cover/

# Translations
*.mo
*.pot

# Django stuff:
*.log
local_settings.py
db.sqlite3
db.sqlite3-journal

# Flask stuff:
instance/
.webassets-cache

# Scrapy stuff:
.scrapy

# Sphinx documentation
docs/_build/

# PyBuilder
.pybuilder/
target/

# Jupyter Notebook
.ipynb_checkpoints

# IPython
profile_default/
ipython_config.py

# pyenv
#   For a library or package, you might want to ignore these files since the code is
#   intended to run in multiple environments; otherwise, check them in:
# .python-version

# pipenv
#   According to pypa/pipenv#598, it is recommended to include Pipfile.lock in version control.
#   However, in case of collaboration, if having platform-specific dependencies or dependencies
#   having no cross-platform support, pipenv may install dependencies that don't work, or not
#   install all needed dependencies.
#Pipfile.lock

# UV
#   Similar to Pipfile.lock, it is generally recommended to include uv.lock in version control.
#   This is especially recommended for binary packages to ensure reproducibility, and is more
#   commonly ignored for libraries.
#uv.lock

# poetry
#   Similar to Pipfile.lock, it is generally recommended to include poetry.lock in version control.
#   This is especially recommended for binary packages to ensure reproducibility, and is more
#   commonly ignored for libraries.
#   https://python-poetry.org/docs/basic-usage/#commit-your-poetrylock-file-to-version-control
#poetry.lock
#poetry.toml

# pdm
#   Similar to Pipfile.lock, it is generally recommended to include pdm.lock in version control.
#   pdm recommends including project-wide configuration in pdm.toml, but excluding .pdm-python.
#   https://pdm-project.org/en/latest/usage/project/#working-with-version-control
#pdm.lock
#pdm.toml
.pdm-python
.pdm-build/

# pixi
#   Similar to Pipfile.lock, it is generally recommended to include pixi.lock in version control.
#pixi.lock
#   Pixi creates a virtual environment in the .pixi directory, just like venv module creates one
#   in the .venv directory. It is recommended not to include this directory in version control.
.pixi

# PEP 582; used by e.g. github.com/David-OConnor/pyflow and github.com/pdm-project/pdm
__pypackages__/

# Celery stuff
celerybeat-schedule
celerybeat.pid

# SageMath parsed files
*.sage.py

# Environments
.env
.envrc
.venv
env/
venv/
ENV/
env.bak/
venv.bak/

# Spyder project settings
.spyderproject
.spyproject

# Rope project settings
.ropeproject

# mkdocs documentation
/site

# mypy
.mypy_cache/
.dmypy.json
dmypy.json

# Pyre type checker
.pyre/

# pytype static type analyzer
.pytype/

# Cython debug symbols
cython_debug/

# PyCharm
#  JetBrains specific template is maintained in a separate JetBrains.gitignore that can
#  be found at https://github.com/github/gitignore/blob/main/Global/JetBrains.gitignore
#  and can be added to the global gitignore or merged into this file.  For a more nuclear
#  option (not recommended) you can uncomment the following to ignore the entire idea folder.
.idea/

# Abstra
# Abstra is an AI-powered process automation framework.
# Ignore directories containing user credentials, local state, and settings.
# Learn more at https://abstra.io/docs
.abstra/

# Visual Studio Code
#  Visual Studio Code specific template is maintained in a separate VisualStudioCode.gitignore
#  that can be found at https://github.com/github/gitignore/blob/main/Global/VisualStudioCode.gitignore
#  and can be added to the global gitignore or merged into this file. However, if you prefer,
#  you could uncomment the following to ignore the entire vscode folder
 .vscode/

# Ruff stuff:
.ruff_cache/

# PyPI configuration file
.pypirc

# Cursor
#  Cursor is an AI-powered code editor. `.cursorignore` specifies files/directories to
#  exclude from AI features like autocomplete and code analysis. Recommended for sensitive data
#  refer to https://docs.cursor.com/context/ignore-files
.cursorignore
.cursorindexingignore

# Marimo
marimo/_static/
marimo/_lsp/
__marimo__/

config/
odoo_pg_pass
"""
    write_file(path, content, filename=".gitignore")


if __name__ == "__main__":
    path = "/home/agga/Documents/odoo-dev/ica_standard_structure/test"
    version = 18.0
    python = 3.10
    create_gitignore(path)



from odooss.utils import write_file


def create_addons(path: str):
    write_file(path, "", filename="addons/README.md")


def create_readme(path: str):
    content = "# Powered by IdeaCode Academy"
    write_file(path, content, filename="README.md")


if __name__ == "__main__":
    path = "/home/agga/Documents/odoo-dev/ica_standard_structure/test"
    version = 18.0
    python = 3.10
    create_addons(path)
    create_readme(path)



import subprocess

from odooss.utils import write_file


def create_precommit_config(path: str):
    content = """\
repos:
  - repo: https://github.com/astral-sh/ruff-pre-commit
    rev: v0.12.4  # Use latest version
    hooks:
      - id: ruff
        name: Ruff Linter for Odoo
        args: [ --fix, ]# Optional: auto-fix on dev machines
        files: ^addons/

      - id: ruff-format
        name: Ruff Formatter
        files: ^addons/"""
    write_file(path, content, filename=".pre-commit-config.yaml")


def install_precommit(path: str):
    # Change directory to target path and run the commands
    try:
        subprocess.run(["git", "init"], cwd=path, check=True)
        subprocess.run(["pre-commit", "install"], cwd=path, check=True)
        subprocess.run(["pre-commit", "run", "--all-files"], cwd=path, check=True)
        print("pre-commit installed and ran successfully.")
    except subprocess.CalledProcessError as e:
        print(f"Error running pre-commit commands: {e}")


if __name__ == "__main__":
    path = "/home/agga/Documents/odoo-dev/ica_standard_structure/test"
    create_precommit_config(path)



import logging

import click

from .main import create_all

logging.basicConfig(level=logging.INFO)
logger = logging.getLogger(__name__)


@click.group()
def cli():
    pass


@cli.command()
@click.argument("path", type=click.Path())
@click.option("--odoo_version", default="18.0", help="Odoo version to use (e.g. 18.0)")
@click.option("--python", default="3.10", help="Python version to use (e.g. 3.10)")
@click.option("--vcs", default="github", help="use some vcs (e.g. github, gitlab,etc)")
def create(path, odoo_version, python,vcs):
    """Create the Odoo standard folder structure at PATH."""
    logger.info("Please make sure your virtual environment is activated.")
    create_all(path, odoo_version, python,vcs)


if __name__ == "__main__":
    cli()



from .cli import cli
from .folder_structure import (
    create_addons,
    create_docker_compose,
    create_dockerfile,
    create_dockerignore,
    create_gitignore,
    create_readme,
    create_requirement,
    create_ruff,
)
from .precommit import create_precommit_config, install_precommit
from .vcs import create_github_workflow
__all__ = [
    "create_precommit_config",
    "install_precommit",
    "create_ruff",
    "create_requirement",
    "create_addons",
    "cli",
    "create_readme",
    "create_gitignore",
    "create_dockerignore",
    "create_dockerfile",
    "create_docker_compose",
    "create_github_workflow",
]
